#!/bin/bash
APPNAME=deCONZ

#LD_LIBRARY_PATH=../lib
#export LD_LIBRARY_PATH

OPTIONS="--auto-connect=1 --dbg-error=1"
EXTRA_OPTIONS=${DECONZ_OPTS:-}

UPDATE_FW_SCRIPT="/var/tmp/deCONZ-update-firmware.sh"

BASEURL="http://www.dresden-elektronik.de/rpi"

FWURL="${BASEURL}/deconz-firmware"
FWDIR="/home/${USER}/raspbee_firmware"
FWVERSION="0x1a0a0500"
FWFILE="deCONZ_Rpi_${FWVERSION}.bin.GCF"

OTAUDIR="/home/${USER}/otau"

CONFIGROOT="/home/${USER}/.local/share/data"
CONFIGDIR1="${CONFIGROOT}/deCONZ"
CONFIGDIR2="${CONFIGROOT}/dresden-elektronik/deCONZ"

#check if deCONZ already running
pidof deCONZ
if [ $? -eq "0" ]; then
	echo "deCONZ already running"
	exit 1
fi

# check if configuration move to new location is needed
if [ ! -d "$CONFIGDIR2" ]; then

	if [ -d "$CONFIGDIR1" ]; then
		echo "copy configuration to ${CONFIGDIR2}"
		mkdir -p $CONFIGDIR2
		mv $CONFIGDIR1/* $CONFIGDIR2
		rmdir $CONFIGDIR1
		ln -s $CONFIGDIR2 $CONFIGDIR1
	else
		echo "${CONFIGDIR1} does not exist"
	fi
fi

# create firmware directory if not exist
if [ ! -d "$FWDIR" ]; then
	mkdir -p $FWDIR
fi

# sleep for some seconds to give the system the chance to connect to the internet 
sleep 10

# if firmware file exist and also a .md5 check if it's valid
# if verification fails it will be deleted and redownloaded
if [ -e "${FWDIR}/${FWFILE}.md5" ]; then
	cd $FWDIR
	md5sum --check "${FWFILE}.md5"

	if [ $? -ne 0 ]; then
		rm -f ${FWDIR}/${FWFILE}
		rm -f ${FWDIR}/${FWFILE}.md5
	fi

	cd -
fi

# check if the RaspBee firmware file exist and if not redownload it
if [ ! -e "${FWDIR}/${FWFILE}" ]; then
	rm -f ${FWURL}/${FWFILE}.md5
	wget --directory-prefix=${FWDIR} ${FWURL}/${FWFILE}.md5
	wget --directory-prefix=${FWDIR} ${FWURL}/${FWFILE}
fi

# create otau directory if not exist
if [ ! -d "$OTAUDIR" ]; then
    mkdir -p ${OTAUDIR}
fi

deCONZ-otau-dl.sh &
DLPID=$!

function finish {
	echo "shutdown deCONZ"
	echo "resetting RaspBee"
	kill -9 $DLPID
	sudo GCFFlasher -r
}

trap finish EXIT

# check for setcap / getcap
dpkg -l libcap2-bin > /dev/null 2>&1

if [ $? -ne 0 ]; then
    sudo apt-get install --yes libcap2-bin
fi

while [ 1 ]
do
	# provide service on port 80
	which setcap

	if [ $? -eq "0" ]; then
		# assume getcap also exists
		caps=`getcap /usr/bin/deCONZ`

		if [[ "$caps" == *cap_net_bind_service+ep* ]]; then
			echo capabilities ok
		else
			sudo setcap cap_net_bind_service=+ep  /usr/bin/deCONZ
			echo set capability to open ports lower 1024
		fi

		OPTIONS+=" --http-port=80"
	else
		OPTIONS+=" --http-port=8080"
	fi

	$APPNAME $OPTIONS $EXTRA_OPTIONS
	RC=$?
	echo "deCONZ - exit code ${RC}"

	case "$RC" in
	0) # Normal exit
		exit 0
		;;
	40) # Download and install updates
		deCONZ-update.sh
		;;
	41) # Restart this script in a new process
		exec $@
		;;
	42) # Download and install updates (beta channel)
		deCONZ-update.sh -c beta
		;;
	43) # Restart system
		sudo reboot
		exit 0
		;;
	44) # Shutdown system
		sudo halt
		exit 0
		;;
	45) # Download and install updates (alpha channel)
		# not yet supported just do a beta channel update
		# deCONZ-update.sh -c alpha
		deCONZ-update.sh -c beta
		;;
	46) # Update firmware only
		if [ -e "$UPDATE_FW_SCRIPT" ]; then
			sh $UPDATE_FW_SCRIPT
			# cleanup
			echo "delete $UPDATE_FW_SCRIPT"
			rm $UPDATE_FW_SCRIPT
		else
			echo "$UPDATE_FW_SCRIPT not found"
		fi
		;;
	126) # Permission problem or command is not executable
		finish
		exit 1
		;;
	127) # Command not found (try to install)
		deCONZ-update.sh
		if [ $? -ne 0 ]; then
			finish
			exit 1
		fi
		;;
	130) # CTRL-C, SIGINT (128 + 2)
		finish
		exit 0
		;;
	137) # SIGKILL (128 + 9)
		finish
		exit 0
		;;
	139) # SEGV (128 + 11)
		;;
	143) # SIGTERM (128 + 15)
		finish
		exit 0
		;;
	*) echo "exit code $RC not handled"
		;;
	esac
done
